@@ -45,9 +45,17 @@ void Trace();
 
 double d_pgumbel(double q, double loc, double scale, int lower_tail)
 {
+  if(ISNAN(q)) // true for NA and NaN
+    return NA_REAL;
+  if(q == R_PosInf) 
+    q = 1.;
+  else if(q == R_NegInf)
+    q = 0.;
+  else {
     q = (q - loc) / scale;
     q = exp( -exp( -q));
-    return !lower_tail ? 1 - q : q;
+  }
+  return !lower_tail ? 1 - q : q;
 }
 
 void pgumbel(double *q, int *nq, double *loc, double *scale,
@@ -63,9 +71,17 @@ void pgumbel(double *q, int *nq, double *loc, double *scale,
 double d_pgumbel2(double q, double loc, double scale, int lower_tail)
 // this is (partly) redundant since d_pgumbel2(q) = 1 - d_pgumbel(-q)
 {
+  if(ISNAN(q)) // true for NA and NaN
+    return NA_REAL;
+  if(q == R_PosInf) 
+    q = 1;
+  else if(q == R_NegInf)
+    q = 0;
+  else {
     q = (-q - loc) / scale;
     q = exp(-exp(-q));
-    return !lower_tail ? q : 1 - q;
+  }
+  return !lower_tail ? q : 1 - q;
 }
 
 void pgumbel2(double *q, int *nq, double *loc, double *scale,
@@ -78,11 +94,19 @@ void pgumbel2(double *q, int *nq, double *loc, double *scale,
 
 double d_pAO(double q, double lambda, int lower_tail)
 {
+  if(ISNAN(q) || ISNAN(lambda)) // true for NA and NaN
+    return NA_REAL;
+  if(q == R_PosInf) 
+    q = 1;
+  else if(q == R_NegInf)
+    q = 0;
+  else {
     if(lambda < 1.0e-6)
-        error("'lambda' has to be positive. lambda = %e was supplied\n",
-	      lambda);
+      error("'lambda' has to be positive. lambda = %e was supplied\n",
+	    lambda);
     q = 1 - R_pow(lambda * exp(q) + 1, -1/lambda);
-    return !lower_tail ? 1 - q : q;
+  }
+  return !lower_tail ? 1 - q : q;
 }
 
 void pAO(double *q, int *nq, double *lambda, int *lower_tail)
@@ -94,18 +118,26 @@ void pAO(double *q, int *nq, double *lambda, int *lower_tail)
 
 double d_plgamma(double eta, double lambda, int lower_tail)
 {
-    double v;
 
+  double v; 
+  if(ISNAN(eta) || ISNAN(lambda)) // true for NA and NaN
+    return NA_REAL;
+  if(eta == R_PosInf) 
+    v = 1;
+  else if(eta == R_NegInf)
+    v = 0;
+  else {
     v = R_pow_di(lambda, -2) * exp(lambda * eta);
     if(lambda < 1.0e-6)
-        v = 1 - pgamma(v, R_pow_di(lambda, -2), /*scale = */ 1,
-		       lower_tail, give_log);
+      v = 1 - pgamma(v, R_pow_di(lambda, -2), /*scale = */ 1,
+		     lower_tail, give_log);
     if(lambda > -1.0e-6)
-        v = pgamma(v, R_pow_di(lambda, -2), /*scale = */ 1,
-		   lower_tail, give_log);
+      v = pgamma(v, R_pow_di(lambda, -2), /*scale = */ 1,
+		 lower_tail, give_log);
     if(lambda >= -1.0e-6 && lambda <= 1.0e-6)
-        v = pnorm(eta, mu, sigma, lower_tail, give_log);
-    return !lower_tail ? 1 - v : v;
+      v = pnorm(eta, mu, sigma, lower_tail, give_log);
+  }
+  return !lower_tail ? 1 - v : v;
 }
 
 void plgamma(double *q, int *nq, double *lambda, int *lower_tail)
@@ -120,9 +152,14 @@ void plgamma(double *q, int *nq, double *lambda, int *lower_tail)
 
 double d_dgumbel(double x, double loc, double scale, int give_log)
 {
-    x = (x - loc) / scale;
-    x = -exp(-x) - x - log(scale);
-    return give_log ? x : exp(x);
+  if(ISNAN(x)) // true for NA and NaN
+    return NA_REAL;
+  if(x == R_PosInf || x == R_NegInf)
+    // if(x == INFINITE || x == -INFINITE) // seems to work as well.
+    return 0; // this special case needs to be handled separately 
+  x = (x - loc) / scale;
+  x = -exp(-x) - x - log(scale);
+  return give_log ? x : exp(x);
 }
 
 void dgumbel(double *x, int *nx, double *loc, double *scale,
@@ -135,9 +172,13 @@ void dgumbel(double *x, int *nx, double *loc, double *scale,
 
 double d_dgumbel2(double x, double loc, double scale, int give_log)
 {
-    x = (-x - loc) / scale;
-    x = -exp(-x) - x - log(scale);
-    return give_log ? x : exp(x);
+  if(ISNAN(x)) // true for NA and NaN
+    return NA_REAL;
+  if(x == R_PosInf || x == R_NegInf)
+    return 0;
+  x = (-x - loc) / scale;
+  x = -exp(-x) - x - log(scale);
+  return give_log ? x : exp(x);
 }
 
 void dgumbel2(double *x, int *nx, double *loc, double *scale,
@@ -150,11 +191,15 @@ void dgumbel2(double *x, int *nx, double *loc, double *scale,
 
 double d_dAO(double eta, double lambda, int give_log)
 {
-    if(lambda < 1.0e-6)
-        error("'lambda' has to be positive. lambda = %e was supplied\n",
-	      lambda);
-    eta -= (1 + 1 / lambda) * log(lambda * exp(eta) + 1);
-    return give_log ? eta : exp(eta);
+  if(ISNAN(eta) || ISNAN(lambda)) // true for NA and NaN
+    return NA_REAL;
+  if(eta == R_PosInf || eta == R_NegInf)
+    return 0;
+  if(lambda < 1.0e-6)
+    error("'lambda' has to be positive. lambda = %e was supplied\n",
+	  lambda);
+  eta -= (1 + 1 / lambda) * log(lambda * exp(eta) + 1);
+  return give_log ? eta : exp(eta);
 }
 
 void dAO(double *x, int *nx, double *lambda, int *give_log)
@@ -166,13 +211,18 @@ void dAO(double *x, int *nx, double *lambda, int *give_log)
 
 double d_dlgamma(double x, double lambda, int give_log)
 {
-    double q_2;
+  if(ISNAN(x) || ISNAN(lambda)) // true for NA and NaN
+    return NA_REAL;
+  if(x == R_PosInf || x == R_NegInf)
+    return 0; 
+  if(lambda < 1.0e-5 && lambda > -1.0e-5) // lambda close to zero
+    return dnorm(x, mu, sigma, give_log); 
 
-    q_2 = R_pow_di(lambda, -2);
-    x *= lambda;
-    x = log(fabs(lambda)) + q_2 * log(q_2) -
-        lgammafn(q_2) + q_2 * (x - exp(x));
-    return !give_log ? exp(x) : x;
+  double q_2 = R_pow_di(lambda, -2);
+  x *= lambda;
+  x = log(fabs(lambda)) + q_2 * log(q_2) -
+    lgammafn(q_2) + q_2 * (x - exp(x));
+  return !give_log ? exp(x) : x;
 }
 
 void dlgamma(double *x, int *nx, double *lambda, int *give_log)
@@ -187,10 +237,22 @@ void dlgamma(double *x, int *nx, double *lambda, int *give_log)
 
 double d_glogis(double x)
 {
-// Gradient of dlogis(x) wrt. x
-    x = exp(-x);
-    return 2 * x * x * R_pow_di(1 + x, -3) - x *
-	R_pow_di(1 + x, -2);
+  // Gradient of dlogis(x) wrt. x
+  if(ISNAN(x)) // true for NA and NaN
+    return NA_REAL;
+  if(x == R_PosInf || x == R_NegInf)
+    // if(x == INFINITE || x == -INFINITE) // seems to work as well.
+    return 0; // this special case needs to be handled separately 
+
+  /* Store the sign of x, compute the gradient for the absolute value
+     and restore the sign. This is needed to avoid exp(LARGE) to blow
+     up and the function to return NaN.
+  */
+  int sign = x > 0; //could use fsign() instead...
+  x = exp(-fabs(x));
+  x = 2 * x * x * R_pow_di(1 + x, -3) - x *
+    R_pow_di(1 + x, -2);
+  return sign ? x : -x;
 }
 
 
@@ -202,6 +264,12 @@ void glogis(double *x, int *nx)
 }
 
 double d_gnorm(double x) {
+  
+  if(ISNAN(x)) // true for NA and NaN
+    return NA_REAL;
+  if(x == INFINITY || x == -INFINITY)
+    return 0;
+  else
     return -x * dnorm(x, mu, sigma, give_log);
 }
 
@@ -215,12 +283,17 @@ void gnorm(double *x, int *nx)
 
 double d_gcauchy(double x)
 {
-    return x = -2 * x / M_PI * R_pow_di(1 + x * x, -2);
+  if(ISNAN(x)) // true for NA and NaN
+    return NA_REAL;
+  if(x == R_PosInf || x == R_NegInf)
+    return 0; 
+  
+  return x = -2 * x / M_PI * R_pow_di(1 + x * x, -2);
 }
 
 void gcauchy(double *x, int *n)
 {
-// Gradient of plogis(x) wrt. x
+// Gradient of dcauchy(x) wrt. x
     int i;
     for(i = 0; i < *n; i++)
 	x[i] = d_gcauchy(x[i]);
@@ -228,11 +301,17 @@ void gcauchy(double *x, int *n)
 
 double d_ggumbel(double x)
 {
-    double eq;
+  if(ISNAN(x)) // true for NA and NaN
+    return NA_REAL;
+  if(x == R_PosInf || x == R_NegInf)
+    return 0; 
 
-    x = exp(-x);
-    eq = exp(-x);
-    return -eq * x + eq * x * x;
+  x = exp(-x);
+  if(x == INFINITY)
+    return 0;
+  
+  double eq = exp(-x);
+  return -eq * x + eq * x * x;
 }
 
 void ggumbel(double *x, int *nx)
@@ -253,15 +332,22 @@ void ggumbel2(double *x, int *nx)
     int i;
     for(i = 0; i < *nx; i++)
 	x[i] = -d_ggumbel(-x[i]);
-    // or x[i] = d_ggumbel2(x);
+    // or x[i] = d_ggumbel2(x[i]);
 }
 
 double d_gAO(double eta, double lambda)
 {
-    double lex;
-    lex = lambda * exp(eta);
-    return d_dAO(eta, lambda, give_log) *
-	(1 - (1 + 1/lambda) * lex / (1 + lex));
+  if(ISNAN(eta) || ISNAN(lambda)) // true for NA and NaN
+    return NA_REAL;
+  if(eta == R_PosInf || eta == R_NegInf)
+    return 0; 
+
+  double lex = lambda * exp(eta);
+  if(lex == R_PosInf || lex == 0)
+    return 0.0;
+  double y = d_dAO(eta, lambda, give_log);
+  
+  return y == 0 ? 0.0 : y * (1 - (1 + 1/lambda) * lex / (1 + lex)); 
 }
 
 void gAO(double *x, int *nx, double *lambda)
@@ -273,8 +359,24 @@ void gAO(double *x, int *nx, double *lambda)
 
 double d_glgamma(double x, double lambda)
 {
-    return (1 - exp(lambda * x)) / lambda *
-	d_dlgamma(x, lambda, give_log);
+  if(ISNAN(x) || ISNAN(lambda)) // true for NA and NaN
+    return NA_REAL;
+  if(x == R_PosInf || x == R_NegInf)
+    return 0; 
+  if(lambda < 1.0e-5 && lambda > -1.0e-5) // lambda close to zero
+    return -x * dnorm(x, mu, sigma, give_log);
+  
+  double z = exp(lambda * x);
+  if(z == R_PosInf || z == 0)
+    return 0.0;
+  double y = d_dlgamma(x, lambda, give_log);
+  return y <= 0 ? 0.0 : y * (1 - exp(lambda * x)) / lambda; 
+  // Equivalent to:
+  /* if(y <= 0) 
+     return 0.0;
+     else 
+     return y * (1 - exp(lambda * x)) / lambda;
+  */
 }
 
 void glgamma(double *x, int *nx, double *lambda)
@@ -894,3 +996,4 @@ double maxAbs(double *x, int nx)
     }
     return cmax;
 }
+
